@@ -54,12 +54,12 @@ test_that("translate sql server -> PostgreSQL date diff (hour, minute, second)",
     targetDialect = "postgresql"
   )
   expect_equal_ignore_spaces(sql, "SELECT (EXTRACT(EPOCH FROM (drug_exposure_end_datetime - drug_exposure_start_datetime)) / 3600) FROM drug_exposure;")
-  
+
   sql <- translate("SELECT DATEDIFF(minute,drug_exposure_start_datetime,drug_exposure_end_datetime) FROM drug_exposure;",
     targetDialect = "postgresql"
   )
   expect_equal_ignore_spaces(sql, "SELECT (EXTRACT(EPOCH FROM (drug_exposure_end_datetime - drug_exposure_start_datetime)) / 60) FROM drug_exposure;")
-  
+
   sql <- translate("SELECT DATEDIFF(second,drug_exposure_start_datetime,drug_exposure_end_datetime) FROM drug_exposure;",
     targetDialect = "postgresql"
   )
@@ -282,5 +282,7 @@ test_that("translate sql server -> postgresql ALTER TABLE ADD CONSTRAINT", {
   expect_equal_ignore_spaces(sql, "ALTER TABLE cdm.MEASUREMENT ADD CONSTRAINT xpk_MEASUREMENT PRIMARY KEY (measurement_id);")
 })
 
-
-
+test_that("translate sql server -> postgresql ALTER TABLE ALTER COLUMN", {
+  sql <- translate("ALTER TABLE my_table ALTER COLUMN a BIGINT;", targetDialect = "postgresql")
+  expect_equal_ignore_spaces(sql, "ALTER TABLE my_table ALTER COLUMN a TYPE BIGINT;")
+})
